@@ -16,7 +16,7 @@
  ** 2003-Jun-24: PLD: Added subject parsing
  **
 
- */
+*/
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -202,19 +202,19 @@ int MIMEH_set_doubleCR( int level )
 
 
 /*-----------------------------------------------------------------\
- Function Name	: MIMEH_set_headerfix
- Returns Type	: int
- 	----Parameter List
-	1. int level , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: MIMEH_set_headerfix
+  Returns Type	: int
+  ----Parameter List
+  1. int level , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int MIMEH_set_headerfix( int level )
 {
@@ -747,11 +747,11 @@ int MIMEH_are_headers_RFC822( char *headers )
 			if (p > lc_headers)
 			{
 				if ((*(p-1) == '\n')||(*(p-1) == '\r'))
-				hitcount++;
+					hitcount++;
 			} else if (p == lc_headers) hitcount++;
 		}
 	}
-	
+
 	if (lc_headers != NULL) free(lc_headers);
 
 	return hitcount;
@@ -969,32 +969,32 @@ int MIMEH_strip_comments( char *input )
 
 
 /*-----------------------------------------------------------------\
- Function Name	: MIMEH_fix_header_mistakes
- Returns Type	: int
- 	----Parameter List
-	1. char *data , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
---------------------------------------------------------------------
- Comments:
+  Function Name	: MIMEH_fix_header_mistakes
+  Returns Type	: int
+  ----Parameter List
+  1. char *data , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
 
-	Some headers are broken in their wrapping, ie, they fail to
-	put a leading space at the start of the next wrapped data line; ie
+Some headers are broken in their wrapping, ie, they fail to
+put a leading space at the start of the next wrapped data line; ie
 
 Content-Transfer-Encoding: quoted-printable
 Content-Disposition: attachment;
 filename="yxnjjhyk.xml"
-	
-	Which causes normal header processing to not locate the filename.
 
-	This function will see if there are any lines with a trailing ; that
-	do not have a leading space on the next line and subsequently replace
-	the \r\n chars after the ; with blanks, effectively pulling the line up
- 
+Which causes normal header processing to not locate the filename.
+
+This function will see if there are any lines with a trailing ; that
+do not have a leading space on the next line and subsequently replace
+the \r\n chars after the ; with blanks, effectively pulling the line up
+
 --------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int MIMEH_fix_header_mistakes( char *data )
 {
@@ -1022,7 +1022,7 @@ int MIMEH_fix_header_mistakes( char *data )
 			nonblank_detected = 0;
 		} else {
 			/** If the ; isn't immediately followed by a \n or \r, then search till
-			  ** the end of the line to see if all the chars are blank **/
+			 ** the end of the line to see if all the chars are blank **/
 
 			while ((*q != '\0')||(*q != '\r')||(*q != '\n')) {
 				switch (*q) {
@@ -1049,7 +1049,7 @@ int MIMEH_fix_header_mistakes( char *data )
 		} /** if nonblank_detected == 1 **/
 
 		/** if we had nothing but blanks till the end of the 
-		  ** line, then we need to pull up the next line **/
+		 ** line, then we need to pull up the next line **/
 		if (*q != '\0') {
 			DMIMEH LOGGER_log("%s:%d:MIMEH_fix_header_mistakes:DEBUG: Line needs fixing",FL);
 			*q = ' ';
@@ -1062,7 +1062,7 @@ int MIMEH_fix_header_mistakes( char *data )
 	} /** while looking for more ';' chars **/
 
 	DMIMEH LOGGER_log("%s:%d:MIMEH_fix_header_mistakes:DEBUG: Done",FL);
-		
+
 	return result;
 }
 
@@ -1081,12 +1081,12 @@ int MIMEH_read_headers( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 	int linesize=0;
 	int totalsize_original=0;
 	int result = 0;
-	int firstline = 1;
+	//int firstline = 1;
 	int search_count=0;
 	char *tmp;
 	char *tmp_original;
 	char *fget_result = NULL;
-	char *headerline_end;
+	//char *headerline_end;
 	char *p;
 	char *linestart;
 	char *lineend;
@@ -1094,20 +1094,20 @@ int MIMEH_read_headers( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 	int is_RFC822_headers=0;	// 20040208-1335:PLD: Added to give an indication if the headers are RFC822 like; used in conjunction with the header_longsearch for pulling apart qmail bouncebacks.
 
 	/**
-			Lets start the ugly big fat DO loop here so that we can, if needed
-			search until we find headers which are actually valid.  Personally
-			I hate this - but people want it in order to detect malformed 
-			(deliberate or otherwise) emails.  It'd be nice if for once in the
-			software world someone actually enforced standards rather than trying
-			to be overly intelligent about interpreting data.  
-	**/
+	  Lets start the ugly big fat DO loop here so that we can, if needed
+	  search until we find headers which are actually valid.  Personally
+	  I hate this - but people want it in order to detect malformed 
+	  (deliberate or otherwise) emails.  It'd be nice if for once in the
+	  software world someone actually enforced standards rather than trying
+	  to be overly intelligent about interpreting data.  
+	 **/
 
 	if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_read_headers:DEBUG: File position = %ld [0x%0X]"
 			,FL
 			,FFGET_ftell(f)
 			,FFGET_ftell(f)
 			);
-	
+
 	do {
 
 		search_count++;
@@ -1127,13 +1127,13 @@ int MIMEH_read_headers( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 			else if (strstr(linestart,"\r\r")) hinfo->crcr_count++;
 			else if (strchr(linestart,'\n')) hinfo->lf_count++;
 
-			if (MIMEH_DNORMAL)LOGGER_log("%s:%d:MIMEH_read_headers: Data In=[sz=%d:tb=%d:mem=%p]'%s'",FL, linesize, f->trueblank, glb.headerline, buffer);
+			if (MIMEH_DNORMAL)LOGGER_log("%s:%d:MIMEH_read_headers: [CRLF=%d, CRCR=%d, LF=%d] Data In=[sz=%d:tb=%d:mem=%p]'%s'",FL, hinfo->crlf_count, hinfo->crcr_count, hinfo->lf_count, linesize, f->trueblank, glb.headerline, buffer);
 
 			// If we are being told to copy the input data to an output file
 			//		then do so here (this is for the originals)
 			if ((glb.original_header_save_to_file > 0)&&(glb.original_header_file != NULL))
 			{
-				if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_read_headers:DEBUG: saving to file...",FL);
+				//if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_read_headers:DEBUG: saving to file...",FL);
 				fprintf(glb.original_header_file,"%s",linestart);
 			}
 
@@ -1168,7 +1168,7 @@ int MIMEH_read_headers( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 
 
 			/** Normal processing of the headers now starts. **/
-			if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_read_headers:DEBUG: realloc'ing dataspace",FL); 
+			//if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_read_headers:DEBUG: realloc'ing dataspace",FL); 
 			tmp = realloc(glb.headerline, totalsize+linesize+1);
 			if (tmp == NULL)
 			{
@@ -1184,7 +1184,7 @@ int MIMEH_read_headers( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 				glb.headerline = tmp;
 				totalsize = linesize;
 				PLD_strncpy(glb.headerline, linestart, (linesize +1));
-				headerline_end = glb.headerline +totalsize;
+				//headerline_end = glb.headerline +totalsize;
 			} // If the global headerline is currently NULL
 			else
 			{
@@ -1271,7 +1271,7 @@ int MIMEH_read_headers( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 				FFGET_SDL_MODE = 0;
 			} // FFGET_doubleCR test
 
-			firstline = 0;
+			//firstline = 0;
 		} // While reading more headers from the source file.
 
 
@@ -1285,16 +1285,16 @@ int MIMEH_read_headers( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 		{
 			if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIME_read_headers:ERROR: FFGET module ran out of input while reading headers",FL);
 			/** If we're meant to be saving the headers, we better do that now, even though we couldn't
-			  ** read everything we wanted to **/
+			 ** read everything we wanted to **/
 			if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIME_read_headers:DEBUG: save_headers=%d totalsize=%d headerline=%s", FL, glb.save_headers, totalsize, glb.headerline);
 
-				if ((glb.save_headers)&&(glb.headerline))
-				{
-					if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIME_read_headers:DEBUG: Saving header line.",FL);
-					MIMEH_fix_header_mistakes( glb.headerline );
-					MDECODE_decode_ISO( glb.headerline, totalsize  );
-					fprintf(glb.header_file,"%s",glb.headerline);
-				}
+			if ((glb.save_headers)&&(glb.headerline))
+			{
+				if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIME_read_headers:DEBUG: Saving header line.",FL);
+				MIMEH_fix_header_mistakes( glb.headerline );
+				MDECODE_decode_ISO( glb.headerline, totalsize  );
+				fprintf(glb.header_file,"%s",glb.headerline);
+			}
 
 			result = -1;
 		} else {
@@ -1305,7 +1305,7 @@ int MIMEH_read_headers( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 				if (is_RFC822_headers == 0)
 				{
 					/** If not RFC822 headers, then clean up everything we allocated in here **/
-					DMIMEH LOGGER_log("%s:%d:MIME_read_headers:DEBUG: No RFC822 headers detected, cleanup.");
+					DMIMEH LOGGER_log("%s:%d:MIME_read_headers:DEBUG: No RFC822 headers detected, cleanup.", FL);
 					MIMEH_headers_cleanup();
 				}
 			}
@@ -1591,7 +1591,7 @@ int MIMEH_parse_header_parameter( struct MIMEH_header_info *hinfo,  char *data,
 
 	// Look for the search string we're after (ie, filename, name, location etc)
 	if (strncmp(hl,searchstr,strlen(searchstr))==0) p = hl; else p = NULL;
-//	p = strstr (hl, searchstr); //TESTING 
+	//	p = strstr (hl, searchstr); //TESTING 
 	if (p != NULL)
 	{
 		char *string = NULL;
@@ -1975,12 +1975,13 @@ int MIMEH_parse_contenttype( char *header_name, char *header_value, struct MIMEH
 
 	DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: Start",FL);
 
-	p = strstr(header_name,"content-type");
+	p = PLD_strstr(header_name,"content-type",1);
 	if (p != NULL)
 	{
 		DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype: Content-type string found in header-name",FL);
+		DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype: Working string: '%s'",FL, header_value);
 
-/** 20041216-1106:PLD: Increase our sanity **/
+		/** 20041216-1106:PLD: Increase our sanity **/
 		hinfo->sanity++;
 		PLD_strlower(  header_value );
 		PLD_strlower(  header_value );
@@ -2019,6 +2020,8 @@ int MIMEH_parse_contenttype( char *header_name, char *header_value, struct MIMEH
 		}
 		else hinfo->content_type = _CTYPE_UNKNOWN;
 
+		DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: Content-type set to '%d'",FL, hinfo->content_type);
+
 		/** Is there an x-mac-type|creator parameter? **/
 		if ((strstr(header_value,"x-mac-type="))&&(strstr(header_value,"x-mac-creator=")))
 		{
@@ -2037,6 +2040,7 @@ int MIMEH_parse_contenttype( char *header_name, char *header_value, struct MIMEH
 
 		// Copy the string to our content-type string storage field
 		p = header_value;
+		DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: Searching for name=value pairs in '%s'", FL, p);
 		if (p != NULL)
 		{
 			char *c = p;
@@ -2069,14 +2073,14 @@ int MIMEH_parse_contenttype( char *header_name, char *header_value, struct MIMEH
 			while ( param != NULL )
 			{
 				/**
-				  **
-				  ** The Process of decoding our line....
-				  **	. While not end of the line...
-				  **		. Remove whitespace
-				  **		. test for 'name'
-				  **		. test for 'boundary'
-				  **		. Move to next char after parameter values
-				  ** 
+				 **
+				 ** The Process of decoding our line....
+				 **	. While not end of the line...
+				 **		. Remove whitespace
+				 **		. test for 'name'
+				 **		. test for 'boundary'
+				 **		. Move to next char after parameter values
+				 ** 
 				 ** Go to the next character after the 'token separator' character
 				 ** and then proceed to absorb any excess white space.
 				 ** Once we've stopped at a new, non-white character, we can begin
@@ -2144,10 +2148,10 @@ int MIMEH_parse_contenttype( char *header_name, char *header_value, struct MIMEH
 				 ** Look for the MIME Boundary specification in the headers 
 				 **/
 				return_value = MIMEH_parse_header_parameter(hinfo, param, "boundary", hinfo->boundary, sizeof(hinfo->boundary), &data_end_point);
-				DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: Param<=>data_end gap = %d", FL,data_end_point -param);
-				DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: param start pos = '%s'",FL, param);
+				DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: Boundary param<=>data_end gap = %d", FL,data_end_point -param);
+				DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: Boundary param start pos = '%s'",FL, param);
 				if (data_end_point > param) param = data_end_point;
-				DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: param start pos = '%s'",FL, param);
+				DMIMEH LOGGER_log("%s:%d:MIMEH_parse_contenttype:DEBUG: Boundary param start pos = '%s'",FL, param);
 
 				if ( return_value == 0 ) {
 					// Move the parameter search point up to where we stopped 
@@ -2226,7 +2230,7 @@ int MIMEH_parse_contentlocation( char *header_name, char *header_value, struct M
 	p = strstr(header_name,"content-location");
 	if (p)
 	{
-/** 20041216-1108:PLD: Increase our sanity **/
+		/** 20041216-1108:PLD: Increase our sanity **/
 		hinfo->sanity++;
 
 		if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIME_parse_contentlocation:DEBUG: Content Location line found - '%s'\n", FL, header_value);
@@ -2243,7 +2247,7 @@ int MIMEH_parse_contentlocation( char *header_name, char *header_value, struct M
 		{
 			if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIME_parse_contentlocation:DEBUG: filename = %s\n", FL, p);
 			snprintf(hinfo->name, sizeof(hinfo->name),"%s",p);
-			snprintf(hinfo->filename, sizeof(hinfo->name),"%s",p);
+			snprintf(hinfo->filename, sizeof(hinfo->filename),"%s",p);
 			FNFILTER_filter(hinfo->filename, _MIMEH_FILENAMELEN_MAX);
 			SS_push(&(hinfo->ss_filenames), hinfo->filename, strlen(hinfo->filename));
 
@@ -2297,7 +2301,7 @@ int MIMEH_parse_contenttransferencoding( char *header_name, char *header_value,
 	p = strstr(header_name,"content-transfer-encoding");
 	if (p)
 	{
- /** 20041216-1107:PLD: Increase our sanity **/
+		/** 20041216-1107:PLD: Increase our sanity **/
 		hinfo->sanity++;
 
 		q = strpbrk(header_value,"\n\r;");
@@ -2408,7 +2412,7 @@ int MIMEH_parse_contentdisposition( char *header_name, char *header_value, struc
 	p = strstr(header_name,"content-disposition");
 	if (p != NULL)
 	{
-/** 20041216-1107:PLD: Increase our sanity **/
+		/** 20041216-1107:PLD: Increase our sanity **/
 		hinfo->sanity++;
 
 		// Change p to now point to the header VALUE, p no longer
@@ -2539,7 +2543,6 @@ int MIMEH_parse_generic( char *header_name, char *header_value, struct MIMEH_hea
 	int compare_result = 0;
 	int tlen;
 
-	DMIMEH LOGGER_log("%s:%d:MIMEH_parse_generic:DEBUG: Searching for %s in %s",FL,tokenstr,header_name);
 	/** Sanity check the parameters **/
 	if (hinfo == NULL) return -1;
 	if (tokenstr == NULL) return -1;
@@ -2549,6 +2552,10 @@ int MIMEH_parse_generic( char *header_name, char *header_value, struct MIMEH_hea
 	if (bsize < 1) return -1;
 
 	tlen = strlen(tokenstr);
+//	if (tlen < 1) return 0; // PLD-20100501-1215; empty string premature exit optimisation.
+
+	DMIMEH LOGGER_log("%s:%d:MIMEH_parse_generic:DEBUG: Searching for %s in %s",FL,tokenstr,header_name);
+
 	compare_result = strncmp( header_name, tokenstr, tlen );
 	if (compare_result == 0)
 	{
@@ -2563,28 +2570,28 @@ int MIMEH_parse_generic( char *header_name, char *header_value, struct MIMEH_hea
 				hinfo->sanity++;
 				break;
 		}
-				
+
 	}
 
 	return 0;
 }
 
 /*-----------------------------------------------------------------\
- Function Name	: MIMEH_parse_subject
- Returns Type	: int
- 	----Parameter List
-	1. char *header_name, 
-	2.  char *header_value, 
-	3.  struct MIMEH_header_info *hinfo , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: MIMEH_parse_subject
+  Returns Type	: int
+  ----Parameter List
+  1. char *header_name, 
+  2.  char *header_value, 
+  3.  struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int MIMEH_parse_subject( char *header_name, char *header_value, struct MIMEH_header_info *hinfo )
 {
@@ -2599,111 +2606,142 @@ int MIMEH_parse_subject( char *header_name, char *header_value, struct MIMEH_hea
 
 
 /*-----------------------------------------------------------------\
- Function Name	: MIMEH_parse_date
- Returns Type	: int
- 	----Parameter List
-	1. char *header_name, 
-	2.  char *header_value, 
-	3.  struct MIMEH_header_info *hinfo , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: MIMEH_parse_date
+  Returns Type	: int
+  ----Parameter List
+  1. char *header_name, 
+  2.  char *header_value, 
+  3.  struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int MIMEH_parse_date( char *header_name, char *header_value, struct MIMEH_header_info *hinfo )
 {
 	return MIMEH_parse_generic( header_name, header_value, hinfo, "date", hinfo->date, sizeof(hinfo->date) );
 }
+
+
+
 /*-----------------------------------------------------------------\
- Function Name	: MIMEH_parse_from
- Returns Type	: int
- 	----Parameter List
-	1. char *header_name, 
-	2.  char *header_value, 
-	3.  struct MIMEH_header_info *hinfo , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: MIMEH_parse_from
+  Returns Type	: int
+  ----Parameter List
+  1. char *header_name, 
+  2.  char *header_value, 
+  3.  struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int MIMEH_parse_from( char *header_name, char *header_value, struct MIMEH_header_info *hinfo )
 {
 	return MIMEH_parse_generic( header_name, header_value, hinfo, "from", hinfo->from, sizeof(hinfo->from) );
 }
 /*-----------------------------------------------------------------\
- Function Name	: MIMEH_parse_to
- Returns Type	: int
- 	----Parameter List
-	1. char *header_name, 
-	2.  char *header_value, 
-	3.  struct MIMEH_header_info *hinfo , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: MIMEH_parse_to
+  Returns Type	: int
+  ----Parameter List
+  1. char *header_name, 
+  2.  char *header_value, 
+  3.  struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int MIMEH_parse_to( char *header_name, char *header_value, struct MIMEH_header_info *hinfo )
 {
 	return MIMEH_parse_generic( header_name, header_value, hinfo, "to", hinfo->to, sizeof(hinfo->to) );
 }
 /*-----------------------------------------------------------------\
- Function Name	: MIMEH_parse_messageid
- Returns Type	: int
- 	----Parameter List
-	1. char *header_name, 
-	2.  char *header_value, 
-	3.  struct MIMEH_header_info *hinfo , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: MIMEH_parse_messageid
+  Returns Type	: int
+  ----Parameter List
+  1. char *header_name, 
+  2.  char *header_value, 
+  3.  struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int MIMEH_parse_messageid( char *header_name, char *header_value, struct MIMEH_header_info *hinfo )
 {
 	return MIMEH_parse_generic( header_name, header_value, hinfo, "message-id", hinfo->messageid, sizeof(hinfo->messageid) );
 }
 /*-----------------------------------------------------------------\
- Function Name	: MIMEH_parse_received
- Returns Type	: int
- 	----Parameter List
-	1. char *header_name, 
-	2.  char *header_value, 
-	3.  struct MIMEH_header_info *hinfo , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: MIMEH_parse_received
+  Returns Type	: int
+  ----Parameter List
+  1. char *header_name, 
+  2.  char *header_value, 
+  3.  struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int MIMEH_parse_received( char *header_name, char *header_value, struct MIMEH_header_info *hinfo )
 {
 	return MIMEH_parse_generic( header_name, header_value, hinfo, "received", hinfo->received, sizeof(hinfo->received) );
 }
 
+/*-----------------------------------------------------------------\
+  Date Code:	: 20081216-210524
+  Function Name	: MIMEH_parse_charset
+  Returns Type	: int
+  ----Parameter List
+  1. char *header_name, 
+  2.  char *header_value, 
+  3.  struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
+--------------------------------------------------------------------
+Changes:
+
+\------------------------------------------------------------------*/
+int MIMEH_parse_charset( char *header_name, char *header_value, struct MIMEH_header_info *hinfo )
+{
+	int result = 0;
+
+	result = MIMEH_parse_generic( header_name, header_value, hinfo, "charset", hinfo->charset, sizeof(hinfo->charset) );
+	if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_parse_charset:DEBUG: Charset value = '%s'", FL, hinfo->charset);
+
+	return result;
+}
+
 /*-----------------------------------------------------------------\
   Function Name	: MIMEH_process_headers
   Returns Type	: int
@@ -2723,19 +2761,21 @@ Changes:
 int MIMEH_headers_process( struct MIMEH_header_info *hinfo, char *headers )
 {
 	/** scan through our headers string looking for information that is
-	  ** valid **/
-	char *safeh, *h, *safehl;
+	 ** valid **/
+	char *h, *safehl;
 	char *current_header_position;
 	int headerlength;
 
 	if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_parse_headers:DEBUG: Start [hinfo=%p]\n",FL, hinfo);
 
-	safeh = h = headers;
+	h = headers;
 
 	/** Duplicate the headers for processing - this way we don't 'taint' the
-	** original headers during our searching / altering. **/
+	 ** original headers during our searching / altering. **/
 
 	headerlength = strlen(h);
+	if (headerlength < 1) return 0; // PLD:20100501-1212; Optimisation if there's no data in the string.
+
 	safehl = malloc(sizeof(char) *(headerlength+1));
 	PLD_strncpy(safehl, h, headerlength+1);
 
@@ -2744,6 +2784,8 @@ int MIMEH_headers_process( struct MIMEH_header_info *hinfo, char *headers )
 	MIMEH_strip_comments(h);
 
 	current_header_position = h;
+	DMIMEH LOGGER_log("%s:%d:MIMEH_headers_process:DEBUG: current_header_position = '%s'",FL,current_header_position);
+
 
 	// Searching through the headers, we seek out header 'name:value;value;value' sets,
 	//		Each set is then cleaned up, seperated and parsed.
@@ -2754,11 +2796,17 @@ int MIMEH_headers_process( struct MIMEH_header_info *hinfo, char *headers )
 		char *header_name_end_position;
 		char *header_value_end_position;
 
+		header_name = header_value = header_name_end_position = header_value_end_position = NULL;
+
 		DMIMEH LOGGER_log("%s:%d:MIMEH_headers_process:DEBUG: Processing '%s'",FL,current_header_position);
 
 		/** Tokenise for the header 'name', ie, content-type, subject etc **/
+		while ((*current_header_position==' ')||(*current_header_position=='\t')) current_header_position++; // PLD-20100501-1549; eat leading whitespace.
+
 		header_name = current_header_position;
+		DMIMEH LOGGER_log("%s:%d:MIMEH_headers_process:DEBUG: Header_name position='%p'",FL, header_name);
 		header_name_end_position = strpbrk( header_name, ":\t " );
+		DMIMEH LOGGER_log("%s:%d:MIMEH_headers_process:DEBUG: End position of header is '%s'",FL,header_name_end_position);
 		if (header_name_end_position == NULL)
 		{
 			// We couldn't find a terminating :, so, instead we try to find
@@ -2784,7 +2832,7 @@ int MIMEH_headers_process( struct MIMEH_header_info *hinfo, char *headers )
 		//		have already unfolded them, such that there should only be one header:value
 		//		pairing per 'line'.
 
-		current_header_position = strpbrk( current_header_position, "\n\r");
+		current_header_position = strpbrk( current_header_position, "\n\r"); 
 		if ( current_header_position == NULL )
 		{
 			// Theoretically, this should not happen, as headers are always
@@ -2842,17 +2890,19 @@ int MIMEH_headers_process( struct MIMEH_header_info *hinfo, char *headers )
 				//		occurs multiple times ( at least once per parsing
 
 				PLD_strlower( header_name );
-				MIMEH_parse_subject( header_name, header_value, hinfo );
 				MIMEH_parse_contenttype( header_name, header_value, hinfo );
+				MIMEH_parse_subject( header_name, header_value, hinfo );
 				MIMEH_parse_contenttransferencoding( header_name, header_value, hinfo );
 				MIMEH_parse_contentdisposition( header_name, header_value, hinfo );
 				/** These items aren't really -imperative- to have, but they do
-				  ** help with the sanity checking **/
+				 ** help with the sanity checking **/
 				MIMEH_parse_date( header_name, header_value, hinfo );
 				MIMEH_parse_from( header_name, header_value, hinfo );
 				MIMEH_parse_to( header_name, header_value, hinfo );
 				MIMEH_parse_messageid( header_name, header_value, hinfo );
 				MIMEH_parse_received( header_name, header_value, hinfo );
+				MIMEH_parse_charset( header_name, header_value, hinfo );
+
 
 				if (hinfo->filename[0] == '\0')
 				{
@@ -2907,6 +2957,31 @@ int MIMEH_headers_process( struct MIMEH_header_info *hinfo, char *headers )
 }
 
 
+/*-----------------------------------------------------------------\
+  Date Code:	: 20081124-184934
+  Function Name	: MIMEH_headers_clearcount
+  Returns Type	: int
+  ----Parameter List
+  1. struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
+--------------------------------------------------------------------
+Changes:
+
+\------------------------------------------------------------------*/
+int MIMEH_headers_clearcount( struct MIMEH_header_info *hinfo ) {
+
+	hinfo->crlf_count = 0;
+	hinfo->lf_count = 0;
+	hinfo->crcr_count = 0;
+	return 0;
+
+}
+
 /*-----------------------------------------------------------------\
   Function Name	: MIMEH_get_headers
   Returns Type	: int
@@ -2932,6 +3007,7 @@ int MIMEH_headers_get( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 	hinfo->name[0] = '\0';
 	hinfo->content_type = _CTYPE_UNKNOWN;
 	hinfo->subject[0] = '\0';
+	hinfo->charset[0] = '\0';
 
 	// 20040116-1234:PLD - added to appease valgrind
 	hinfo->content_disposition = 0;
@@ -2941,7 +3017,20 @@ int MIMEH_headers_get( struct MIMEH_header_info *hinfo, FFGET_FILE *f )
 	hinfo->crlf_count=0;
 	hinfo->crcr_count=0;
 	hinfo->lf_count=0;
-	snprintf(hinfo->delimeter,sizeof(hinfo->delimeter),"\r\n");
+
+	if (f->linebreak == (FFGET_LINEBREAK_CR|FFGET_LINEBREAK_LF)) {
+		if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_headers_get:DEBUG: Setting to CRLF based on ffget value of %d", FL, f->linebreak);
+		snprintf(hinfo->delimeter,sizeof(hinfo->delimeter),"\r\n");
+		hinfo->crlf_count = 1;
+	} else if (f->linebreak == FFGET_LINEBREAK_LF) {
+		if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_headers_get:DEBUG: Setting to LF based on ffget value of %d", FL, f->linebreak);
+		snprintf(hinfo->delimeter,sizeof(hinfo->delimeter),"\n");
+		hinfo->lf_count = 1;
+	} else {
+		if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_headers_get:DEBUG: ffget value of %d offered us no guide for the delimeter", FL, f->linebreak);
+	}
+
+
 
 	// Initialise header defects array.
 	hinfo->header_defect_count = 0;
@@ -3035,13 +3124,13 @@ int MIMEH_parse_headers( FFGET_FILE *f, struct MIMEH_header_info *hinfo )
 
 	/** 20041216-1100:PLD: Set the header sanity to zero **/
 	if ( result == 0 ) hinfo->sanity = 0; 
-	
+
 	/** Proceed to read, process and finish headers **/
 	DMIMEH LOGGER_log("%s:%d:MIMEH_parse_headers:DEBUG: Getting headers",FL);
 	if ( result == 0 ) result = MIMEH_headers_get( hinfo, f );
 	DMIMEH LOGGER_log("%s:%d:MIMEH_parse_headers:DEBUG: Processing headers",FL);
 	if ( result == 0 ) result = MIMEH_headers_process( hinfo, glb.headerline );
-	DMIMEH LOGGER_log("%s:%d:MIMEH_parse_headers:DEBUG: Cleanup of headers",FL);
+	DMIMEH LOGGER_log("%s:%d:MIMEH_parse_headers:DEBUG: cleanup of headers",FL);
 	if ( result == 0 ) result = MIMEH_headers_cleanup(); 
 	if (MIMEH_DNORMAL) LOGGER_log("%s:%d:MIMEH_parse_headers:DEBUG: END [F=%p, hinfo=%p, sanity=%d]\n", FL, f, hinfo, hinfo->sanity);
 
@@ -3113,5 +3202,40 @@ int MIMEH_get_defect_count( struct MIMEH_header_info *hinfo )
 }
 
 
+/*-----------------------------------------------------------------\
+  Date Code:	: 20090406-114302
+  Function Name	: MIMEH_read_primary_headers
+  Returns Type	: int
+  ----Parameter List
+  1. char *fname, 
+  2.  struct MIMEH_header_info *hinfo , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
+--------------------------------------------------------------------
+Changes:
+
+\------------------------------------------------------------------*/
+int MIMEH_read_primary_headers( char *fname, struct MIMEH_header_info *hinfo )
+{
+	FFGET_FILE F;
+	FILE *f;
+
+	f = fopen(fname,"r");
+	if (!f) { 
+		LOGGER_log("%s:%d:MIMEH_read_primary_headers:ERROR: Cannot open mailpack '%s' (%s)", FL, fname, strerror(errno));
+		return 0;
+	}
+	FFGET_setstream(&F,f);
+
+	MIMEH_parse_headers(&F, hinfo);
+	fclose(f);
+
+	return 0;
+}
+
 //----------------------END
 
