@@ -1,7 +1,7 @@
 #ifndef __alterMIME__
 #define __alterMIME__
 
-#define LIBAM_VERSION "200811161138"
+#define LIBAM_VERSION "200811241138"
 
 #define AM_RETURN_SIGNED_EMAIL 10
 #define AM_RETURN_B64_ENCODED_EMAIL 12
@@ -25,6 +25,9 @@
 
 struct AM_disclaimer_details {
 
+	// File details
+	long segment_start; // used to remember where the current MIME segment starts in the file.
+
 	// Header details
 
 	int content_type;
@@ -46,6 +49,8 @@ struct AM_disclaimer_details {
 	char *disclaimer_text_plain;
 	char *disclaimer_text_HTML;
 	char *disclaimer_text_b64;
+	char *disclaimer_attachment; 
+
 
 	/** Positional definitions for the HTML and text disclaimers **/
 	char textpos[1024];
@@ -72,20 +77,27 @@ struct AM_globals {
 
 	char ldelimeter[3];
 
+	char *disclaimer_plain_base_fname; // will always hold the primary filename of the plain-text disclaimer. Need this for the charset extensions
 	char *disclaimer_plain;
 	int disclaimer_plain_type;
 
+	char *disclaimer_HTML_base_fname; // will always hold the primary filename of the HTML disclaimer. Need this for the charset extensions
 	char *disclaimer_HTML;
 	int disclaimer_HTML_type;
 
 	char *disclaimer_b64;
 	int disclaimer_b64_type;
 
+	char *disclaimer_attachment;
+
 #ifdef ALTERMIME_PRETEXT
+
 	char *pretext_plain;
 	int pretext_plain_type;
 	char *pretext_HTML;
 	int pretext_HTML_type;
+	char *pretext_b64;
+	int pretext_b64_type;
 
 	int pretext_insert;
 #endif
@@ -113,11 +125,13 @@ int AM_set_debug( int level );
 char *AM_set_disclaimer_b64( char *filename, int disclaimer_type );
 char *AM_set_disclaimer_plain( char *filename, int disclaimer_type );
 char *AM_set_disclaimer_HTML( char *filename, int disclaimer_type );
+char *AM_set_disclaimer_attachment( char *filename, int index );
 
 #ifdef ALTERMIME_PRETEXT
 int AM_set_pretext_plain( char *filename, int disclaimer_type );
 int AM_set_pretext_HTML( char *filename, int disclaimer_type );
 int AM_set_pretext_insert( int level );
+int AM_add_pretext( char *mpackname );
 #endif
 
 int AM_add_disclaimer( char *mpackname );
@@ -129,12 +143,15 @@ int AM_set_force_into_b64( int level );
 int AM_set_multipart_insert( int level );
 int AM_set_nullifyall( int level );
 int AM_set_altersigned( int level );
+int AM_set_pretext( int level );
 int AM_set_header_long_search( int level );
 int AM_base64_encode( char *enc_fname, char *out_fname );
 int AM_attachment_replace( char *mpackname, char *attachmentname, char *new_attachment_name );
 int AM_insert_Xheader( char *fname, char *xheader);
 int AM_alter_header( char *filename, char *header, char *change, int change_mode );
 
+int AM_insert_file( char *fname, int encoding_format, char *headers, char *boundary_to_match );
+
 //int AM_force_into_b64( int level );
 
 #endif
